/**
 * Copyright 2007-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.pool.api;


/**
 * Defines a common pool interface used to manage objects.
 * @param <InstanceType> the type of the object that are managed by the pool
 *        (could be EasyBeansSLSB, etc.)
 * @param <Clue> a clue to retrieve a specific instance in the pool
 * @author Florent Benoit
 */
public interface Pool<InstanceType, Clue> {

    /**
     * Gets an object from the pool.
     * @return an instance of an object with type Type.
     * @throws PoolException if instance cannot be returned.
     */
    InstanceType get() throws PoolException;

    /**
     * Gets an object by using a specific hint.
     * @param clue attribute used to retrieve a given instance
     * @return a specific instance of the resource.
     * @throws PoolException if instance cannot be returned.
     */
    InstanceType get(Clue clue) throws PoolException;

    /**
     * Puts back the instance in the pool so it can be reused.
     * @param instance which will be put back in the pool.
     * @throws PoolException if instance is not released.
     */
    void release(InstanceType instance) throws PoolException;

    /**
     * Discard the instance which is in the pool.
     * @param instance which will be discarded.
     * @throws PoolException if instance is not discarded.
     */
    void discard(InstanceType instance) throws PoolException;

    /**
     * Start the pool.<br>
     * It could create initial instances if specified.
     * @throws PoolException if initialization fails
     */
    void start() throws PoolException;

    /**
     * Stop this pool.
     * @throws PoolException if destroy fails
     */
    void stop() throws PoolException;

    /**
     * @return state of this pool.
     */
    IPoolState getState();

    /**
     * @return state of this pool.
     */
    IPoolConfiguration getPoolConfiguration();

    /**
     * Configure the pool with a given configuration.
     * @param poolConfiguration the given configuration
     */
    void setPoolConfiguration(IPoolConfiguration poolConfiguration);


}
